import 'dart:io';

import 'package:analyzer/diagnostic/diagnostic.dart';

import '../../../foundation.dart';
import '../analyze_reporter.dart';

class GithubReporter extends AnalyzeReporter {
  final void Function(Object? object) printer;

  const GithubReporter(this.printer);

  @override
  String get reporterId => 'github';

  @override
  Future<void> report(Iterable<AnalyzedResult> resultList) async {
    // https://docs.github.com/en/actions/learn-github-actions/workflow-commands-for-github-actions#setting-a-warning-message
    for (final result in resultList) {
      for (final issue in result.issues) {
        final source =
            (issue.location.start.sourceUrl?.path ?? 'unknown source')
                .replaceFirst(Directory.current.path + "/", "");
        final startLine = issue.location.start.line;
        final endLine = issue.location.end.line;
        final startColumn = issue.location.start.column;
        final endColumn = issue.location.end.column;

        final severity = _buildSeverity(issue.severity);
        printer(
          '::$severity file=$source,line=$startLine,endLine=$endLine,col=$startColumn,endColumn=$endColumn,title=${issue.ruleId}::${issue.message}',
        );
      }
    }
  }

  String _buildSeverity(Severity severity) {
    switch (severity) {
      case Severity.error:
        return 'error';
      case Severity.warning:
        return 'warning';
      case Severity.info:
        return 'notice';
    }
  }
}
